@@ -119,6 +119,24 @@ def msg_to_string(msg):
     """
     return array.array("B", msg).tostring().encode("hex")
 
+def is_timeout(ioerror):
+    """
+    True if ioerror can be categorized as a timeout.
+    """
+    try:
+        # all IOerrors returned by pyusb contain
+        # msg, errno, and should be unpackable.
+        err, msg = ioerror
+    except ValueError:
+        # but, sometimes we can't unpack. I don't
+        # know what is raising theses IOErrors
+        # just assume its a timeout, so operation
+        # is retried
+        return True
+    else:
+        return (err == errno.ETIMEDOUT #libusb10
+                or msg == "Connection timed out") #libusb01
+
 def generate_checksum(msg):
     """
     Generate a checksum of the given msg.
@@ -331,18 +349,25 @@ ChannelEvent = message(DIR_IN, "CHANNEL_EVENT", 0x40, "BBB", ["channel_number",
 ChannelStatus = message(DIR_IN, "CHANNEL_STATUS", 0x52, "BB", ["channel_number", "channel_status"])
 ChannelId = message(DIR_IN, "CHANNEL_ID", 0x51, "BHBB", ["channel_number", "device_number", "device_type_id", "man_id"])
 AntVersion = message(DIR_IN, "VERSION", 0x3e, "11s", ["ant_version"])
-Capabilities = message(DIR_IN, "CAPABILITIES", 0x54, "BBBBBx", ["max_channels", "max_networks", "standard_opts", "advanced_opts1", "advanced_opts2"])
+#Capabilities = message(DIR_IN, "CAPABILITIES", 0x54, "BBBBBx", ["max_channels", "max_networks", "standard_opts", "advanced_opts1", "advanced_opts2"])
 SerialNumber = message(DIR_IN, "SERIAL_NUMBER", 0x61, "I", ["serial_number"])
 # Synthetic Commands
 UnimplementedCommand = message(None, "UNIMPLEMENTED_COMMAND", None, None, ["msg_id", "msg_contents"])
 
+# hack, capabilites may be 4 (AP1) or 6 (AP2) bytes
+class Capabilities(message(DIR_IN, "CAPABILITIES", 0x54, "BBBB", ["max_channels", "max_networks", "standard_opts", "advanced_opts1"])):
+
+    @classmethod
+    def unpack_args(cls, packed_args):
+        return super(Capabilities, cls).unpack_args(packed_args[:4])
+
+
 ALL_ANT_COMMANDS = [ UnassignChannel, AssignChannel, SetChannelId, SetChannelPeriod, SetChannelSearchTimeout,
                      SetChannelRfFreq, SetNetworkKey, ResetSystem, OpenChannel, CloseChannel, RequestMessage,
                      SetSearchWaveform, SendBroadcastData, SendAcknowledgedData, SendBurstTransferPacket,
                      StartupMessage, SerialError, RecvBroadcastData, RecvAcknowledgedData, RecvBurstTransferPacket,
                      ChannelEvent, ChannelStatus, ChannelId, AntVersion, Capabilities, SerialNumber ]
 
-
 class ReadData(RequestMessage):
     """
     A phony command which is pushed to request data from client.
@@ -433,7 +458,7 @@ class Core(object):
 
     def pack(self, command):
         """
-        Return an array of byte representing
+        Return an array of bytes representing
         the data which needs to be written to
         hardware to execute the given command.
         """
@@ -466,7 +491,7 @@ class Core(object):
 
     def send(self, command, timeout=100):
         """
-        Execute the given command. Retruns true
+        Execute the given command. Returns true
         if command was written to device. False
         if the device nack'd the write. When
         the method returns false, caller should
@@ -475,21 +500,20 @@ class Core(object):
         msg = self.pack(command)
         if not msg: return True
         _trace.debug("SEND: %s", msg_to_string(msg))
-        # ant protocol states \x00\x00 padding is optiontal.
+        # ant protocol states \x00\x00 padding is optional.
         # libusb01 is quirky when using multiple threads?
         # adding the \00's seems to help with occasional issue
-        # where read can block indefinately until more data
+        # where read can block indefinitely until more data
         # is received.
         msg.extend([0] * 2)
         try:
             self.hardware.write(msg, timeout)
             return True
-        except IOError as (err, msg):
-            if err == errno.ETIMEDOUT: return False #libusb10
-            elif msg == "Connection timed out": return False #libusb01
+        except IOError as err:
+            if is_timeout(err): return False
             else: raise
 
-    def recv(self, timeout=500):
+    def recv(self, timeout=1000):
         """
         A generator which return commands
         parsed from input stream of ant device.
@@ -502,16 +526,15 @@ class Core(object):
                     _trace.debug("RECV: %s", msg_to_string(msg))
                     cmd = self.unpack(msg)
                     if cmd: yield cmd
-            except IOError as (err, msg):
+            except IOError as err:
                 # iteration terminates on timeout
-                if err == errno.ETIMEDOUT: raise StopIteration() #libusb10
-                elif msg == "Connection timed out": raise StopIteration() #libusb01
+                if is_timeout(err): raise StopIteration()
                 else: raise
 
 
 class Session(object):
     """
-    Provides synchrous (blocking) API
+    Provides synchronous (blocking) API
     on top of basic (Core) ANT impl.
     """
 
@@ -565,12 +588,13 @@ class Session(object):
         """
         self._send(ResetSystem(), timeout=.5, retry=5)
         if not self.channels:
+            _log.debug("Querying ANT capabilities")
             cap = self.get_capabilities() 
-            ver = self.get_ant_version()
-            sn = self.get_serial_number()
+            #ver = self.get_ant_version()
+            #sn = self.get_serial_number()
             _log.debug("Device Capabilities: %s", cap)
-            _log.debug("Device ANT Version: %s", ver)
-            _log.debug("Device SN#: %s", sn)
+            #_log.debug("Device ANT Version: %s", ver)
+            #_log.debug("Device SN#: %s", sn)
             self.channels = [Channel(self, n) for n in range(0, cap.max_channels)]
             self.networks = [Network(self, n) for n in range(0, cap.max_networks)]
         self._recv_buffer = [[]] * len(self.channels)
@@ -601,25 +625,36 @@ class Session(object):
         seconds. If retry is non-zero, commands returning
         EAGAIN will be retried. retry also appleis to
         RESET_SYSTEM commands. This method blocks until
-        a repsonse if received from hardware or timeout.
+        a response if received from hardware or timeout.
         Care should be taken to ensure timeout is sufficiently
         large. Care should be taken to ensure timeout is
-        at-least as large a on message period.
+        at least as large as a on message period.
         """
         _log.debug("Executing Command. %s", cmd)
         for t in range(0, retry + 1):
             # invalid to send command while another is running
-            # (execpt for reset system)
+            # (except for reset system)
             assert not self.running_cmd or isinstance(cmd, ResetSystem)
-            # set expiration and event on command. Once self.runnning_cmd
-            # is set access to this command from this tread is invalid 
-            # until event object is set.
-            cmd.expiration = time.time() + timeout if timeout > 0 else None
-            cmd.done = threading.Event()
-            self.running_cmd = cmd
+            # HACK, need to clean this up. not all devices support sending
+            # a response message for ResetSystem, so don't bother waiting for it
+            if not isinstance(cmd, ResetSystem):
+                # set expiration and event on command. Once self.runnning_cmd
+                # is set access to this command from this thread is invalid 
+                # until event object is set.
+                cmd.expiration = time.time() + timeout if timeout > 0 else None
+                cmd.done = threading.Event()
+                self.running_cmd = cmd
+            else:
+                # reset is done without waiting
+                cmd.done = threading.Event()
+                cmd.result = StartupMessage(0)
             # continue trying to commit command until session closed or command timeout 
             while self.running and not cmd.done.is_set() and not self.core.send(cmd):
                 _log.warning("Device write timeout. Will keep trying.")
+            if isinstance(cmd, ResetSystem):
+                # sleep to give time for reset to execute
+                time.sleep(1)
+                cmd.done.set()
             # continue waiting for command completion until session closed
             while self.running and not cmd.done.is_set():
                 if isinstance(cmd, SendBurstData) and cmd.has_more_data:
@@ -632,12 +667,12 @@ class Session(object):
                     if self.core.send(packet): cmd.incr_packet_index()
                 else:
                     cmd.done.wait(1)
-            # cmd.done guarenetees a result is availible
+            # cmd.done guarantees a result is available
             if cmd.done.is_set():
                 try:
                     return cmd.result
                 except AttributeError:
-                    # must have failed, theck if error is retryable
+                    # must have failed, check if error is retryable
                     if t < retry and cmd.is_retryable(cmd.error):
                         _log.warning("Retryable error. %d try(s) remaining. %s", retry - t, cmd.error)
                     else:
@@ -674,7 +709,7 @@ class Session(object):
         """
         Append incoming ack messages to read buffer.
         Append completed burst message to buffer.
-        Filly running command from buffer if data availible.
+        Full run command from buffer if data available.
         """
         # handle update the recv buffers
         try:
@@ -682,14 +717,14 @@ class Session(object):
             # (and buffered if no read is currently running)
             if isinstance(cmd, RecvAcknowledgedData):
                 self._recv_buffer[cmd.channel_number].append(cmd)
-            # burst data double-buffered. it is not made availible to
+            # burst data double-buffered. it is not made available to
             # client until the complete transfer is completed.
             elif isinstance(cmd, RecvBurstTransferPacket):
                 channel_number = 0x1f & cmd.channel_number
                 self._burst_buffer[channel_number].append(cmd)
-                # burst complete, make the complete burst availible for read.
+                # burst complete, make the complete burst available for read.
                 if cmd.channel_number & 0x80:
-                    _log.debug("Burst transfer completed, marking %d packets availible for read.", len(self._burst_buffer[channel_number]))
+                    _log.debug("Burst transfer completed, marking %d packets available for read.", len(self._burst_buffer[channel_number]))
                     self._recv_buffer[channel_number].extend(self._burst_buffer[channel_number])
                     self._burst_buffer[channel_number] = []
             # a burst transfer failed, any data currently read is discarded.
@@ -700,11 +735,11 @@ class Session(object):
         except IndexError:
             _log.warning("Ignoring data, buffers not initialized. %s", cmd)
 
-        # dispatcher data if running command is ReadData and somethign avaiblible
+        # dispatcher data if running command is ReadData and something available
         if self.running_cmd and isinstance(self.running_cmd, ReadData):
             if isinstance(cmd, RecvBroadcastData) and self.running_cmd.data_type == RecvBroadcastData:
                 # read broadcast is unbuffered, and blocks until a broadcast is received
-                # if a broadcast is recieved and nobody is lisening it is discarded.
+                # if a broadcast is received and nobody is listening it is discarded.
                 self._set_result(cmd)
             elif self._recv_buffer[self.running_cmd.channel_number]:
                 if self.running_cmd.data_type == RecvAcknowledgedData:
@@ -714,7 +749,7 @@ class Session(object):
                         self._recv_buffer[self.running_cmd.channel_number].remove(ack_msg)
                         break
                 elif self.running_cmd.data_type in (RecvBurstTransferPacket, ReadData):
-                    # selectin a single entire burst transfer or ACK
+                    # select in a single entire burst transfer or ACK
                     data = []
                     for pkt in list(self._recv_buffer[self.running_cmd.channel_number]):
                         if isinstance(pkt, RecvBurstTransferPacket) or self.running_cmd.data_type == ReadData:
